import json

import google.generativeai as genai

from src.gemini_sdk.promt import GEMINI_BASE_MESSAGE
from src.gemini_sdk.schemas import FullRequestForGeminiSchema, ResponseFromGeminiSchema
from src.schemas import MessagesForSendToWorkersSchema
from src.settings.base import settings


class GoogleHelper:
    def __init__(
        self,
        api_key: str,
        model_name: str,
    ) -> None:
        self.api_key = api_key
        self.model = model_name

        genai.configure(api_key=api_key)
        self._model = genai.GenerativeModel(model_name=model_name)

    @staticmethod
    def _serialize_messages_to_promt(
        chats: MessagesForSendToWorkersSchema,
    ) -> list[dict]:
        messages_for_request = GEMINI_BASE_MESSAGE.copy()

        text_for_request = json.dumps(chats.model_dump_json())

        extend_message = {
            "role": "user",
            "parts": [
                {
                    "text": text_for_request,
                }
            ],
        }

        messages_for_request.append(extend_message)

        return messages_for_request

    @staticmethod
    def _serialize_response_to_json(
            response_text: str,
    ) -> ResponseFromGeminiSchema:
        response = response_text.replace('\n', '')
        response_as_dict = json.loads(response_text)

        return ResponseFromGeminiSchema(**response_as_dict)

    def create_request_ai(
        self,
        messages: MessagesForSendToWorkersSchema,
    ) -> ResponseFromGeminiSchema:
        contents = self._serialize_messages_to_promt(messages)
        response = self._model.generate_content(
            contents=contents
        )

        return self._serialize_response_to_json(response.text)



gemini_helper = GoogleHelper(
    api_key=settings.GEMINI.API_KEY,
    model_name=settings.GEMINI.MODEL_NAME,
)