import asyncio

from pyrogram import Client
from pyrogram.types import Message

from src.core.tg_service.schemas import MessageFromChatSchema
from src.core.tg_service.service import check_user_exists, check_chat_exists, check_chunk_state_and_publish
from src.core.tg_service import utils as api_tg_utils


DATA: dict[int, list[MessageFromChatSchema]] = dict()


lock = asyncio.Lock()

async def message_listener(client: Client, message: Message):
    print("received message")
    if api_tg_utils.check_message_condition(message):
        async with lock:
            await check_user_exists(
                user_pyrogram=message.from_user,
            )
            await check_chat_exists(
                chat_pyrogram=message.chat,
            )

        message_schema = MessageFromChatSchema(
            id=message.id,
            user_id=message.from_user.id,
            chat_id=message.chat.id,
            text=message.text,
            message_time=message.date,
        )

        await check_chunk_state_and_publish(
            data=DATA,
            message_schema=message_schema,
            chat_id=message.chat.id,
        )

