import uuid

from pyrogram.types.user_and_chats import User as PyroUser, Chat as PyroChat

from src.core.database import TgChat, User
from src.core.rabbitmq.connect import message_handler_publisher
from src.core.redis_helper.redis_connect import redis_client
from src.core.tg_service.constants import MESSAGE_CHANG_SIZE
from src.core.tg_service.schemas import UserFromMessageSchema, MessagesForSendToWorkersSchema, MessageFromChatSchema
from src.core.database.connect import db_helper
from src.core.tg_service import crud as tg_crud


async def check_user_exists(
        user_pyrogram: PyroUser,
) -> None:
    user_schema = UserFromMessageSchema(
        username=user_pyrogram.username,
        first_name=user_pyrogram.first_name,
        last_name=user_pyrogram.last_name,
        id=user_pyrogram.id,
    )

    user_model = await redis_client.get(name=str(user_schema.id))
    if user_model:
        return

    async with db_helper.get_async_session_not_closed() as session:
        user_model = await session.get(User, user_schema.id)

        if not user_model:
            await tg_crud.create_new_user(
                session=session,
                **user_schema.model_dump()
            )

    await redis_client.set(name=str(user_schema.id), value=str(True))


async def check_chat_exists(
        chat_pyrogram: PyroChat
) -> None:
    chat = await redis_client.get(name=str(chat_pyrogram.id))

    if chat:
        return

    async with db_helper.get_async_session_not_closed() as session:
        chat = await session.get(TgChat, chat_pyrogram.id)

        if not chat:
            await tg_crud.create_new_chat(
                session=session,
                id=chat_pyrogram.id,
                chat_type=chat_pyrogram.type.value,
                title=chat_pyrogram.title,
            )
    await redis_client.set(name=str(chat_pyrogram.id), value=str(True))


async def check_chunk_state_and_publish(
        data: dict[int, list[MessageFromChatSchema]],
        chat_id: int,
        message_schema: MessageFromChatSchema,
):
    messages_chunk = data.get(chat_id)

    if messages_chunk is None:
        data[chat_id] = [message_schema]

    elif len(messages_chunk) == MESSAGE_CHANG_SIZE:
        slice_id = uuid.uuid4()
        del data[chat_id]
        async with db_helper.get_async_session_not_closed() as session:
            await tg_crud.bulk_insert_messages(
                messages=messages_chunk,
                session=session,
                slice_id=slice_id
            )
        await message_handler_publisher.publish(
            message=MessagesForSendToWorkersSchema(
                messages=messages_chunk,
                slice_id=slice_id
            )
        )
        data[chat_id] = [message_schema]

    else:
        data[chat_id].append(message_schema)


