from pyrogram.types import Message
from pyrogram.enums import ChatType

from src.core.database import TgMessage, User, TgChat


def check_message_condition(
        message: Message,
) -> bool:
    conditions = (
        message.chat.type not in [ChatType.PRIVATE, ChatType.BOT],
        bool(message.from_user),
        bool(message.text),
    )
    return all(conditions)


def create_and_format_message(
        messages: list[TgMessage],
        reason: str,
        chat: TgChat,
        user_model: User,
) -> str:
    def escape_markdown_v2(text: str) -> str:
        escape_chars = '_*[]()~`>#+-=|{}.!'
        return ''.join('\\' + char if char in escape_chars else char for char in text)

    # Формирование информации о пользователе
    username = escape_markdown_v2(user_model.username) if user_model.username else f"ID: {user_model.id}"
    user_link = f"[{username}](tg://user?id={user_model.id})" if user_model.username else f"ID: {user_model.id}"

    # Формирование информации о чате
    chat_title = escape_markdown_v2(chat.title)
    chat_link = (
        f"https://t.me/c/{str(chat.id)[4:]}"
        if str(chat.id).startswith("-100")
        else f"https://t.me/{chat.username}" if chat.username
        else f"Chat ID: {chat.id}"
    )

    # Экранирование причины
    reason_escaped = escape_markdown_v2(reason)

    # Базовый заголовок сообщения
    header = (
        f"🔥 *Найдена успешка!*\n"
        f"👤 *Пользователь:* {user_link}\n"
        f"🐩 *Чат:* [{chat_title}]({chat_link})\n"
        f"📌 *Причина:* {reason_escaped}\n\n"
        f"📝 *Диалог:*\n"
    )

    # Расчет доступной длины для контента
    MAX_LENGTH = 4096
    header_length = len(header)
    available_length = MAX_LENGTH - header_length

    # Формирование блоков сообщений
    message_blocks = []
    current_length = 0

    for msg in messages:
        # Очистка и подготовка текста
        clean_text = msg.text.replace('```', 'ʻʻʻ')  # Заменяем опасные символы
        escaped_text = escape_markdown_v2(clean_text)

        sender_username = escape_markdown_v2(
            msg.user_relationship.username or str(msg.user_id)
        )
        # Формирование блока сообщения
        block = (
            f"**{sender_username}:**\n"
            f"```\n"
            f"{escaped_text}\n"
            f"```\n\n"
        )
        block_length = len(block)

        # Проверка на превышение длины
        if current_length + block_length > available_length:
            remaining_space = available_length - current_length
            if remaining_space > 20:  # Минимальный значимый блок
                truncated_text = escaped_text[:remaining_space - 20] + "..."
                block = (
                    f"**Пользователь:**\n"
                    f"```\n"
                    f"{truncated_text}\n"
                    f"```\n\n"
                    f"_... сообщение обрезано ..._"
                )
                message_blocks.append(block)
            break

        message_blocks.append(block)
        current_length += block_length

    # Сборка финального сообщения
    content = ''.join(message_blocks).strip()
    full_message = header + content

    # Финальная проверка длины
    if len(full_message) > MAX_LENGTH:
        full_message = full_message[:MAX_LENGTH - 17] + "\n```\n...\n```\n_... сообщение обрезано ..._"

    return full_message